@@ -1,4 +1,4 @@
-/* $Id: blast_def.h,v 1.89 2009/05/27 17:39:36 kazimird Exp $
+/* $Id: blast_def.h,v 1.91 2010/07/27 18:24:31 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,6 +116,12 @@ NCBI_XBLAST_EXPORT extern const int kUngappedHSPNumMax;
 NCBI_XBLAST_EXPORT
 void __sfree(void** x);
 
+/** This symbol enables the verbose option in makeblastdb and other BLAST+
+ * search command line applications, as well as the option to submit searches
+ * to the test server in NCBI for remote BLAST searches 
+#define _BLAST_DEBUG 1
+*/
+
 #if 0
 /** Define this symbol to enable debugging APIs in the BlastSeqSrc interface to
  * allow diagnostics/debugging to be performed in the composition based
@@ -225,12 +231,16 @@ typedef struct BlastMaskLoc {
    BlastSeqLoc** seqloc_array; 
 } BlastMaskLoc;
 
+#define DB_MASK_NONE 0   /* no db mask */
+#define DB_MASK_SOFT 1   /* soft db mask */
+#define DB_MASK_HARD 2   /* hard db mask */
+
 /** Structure to hold a sequence. */
 typedef struct BLAST_SequenceBlk {
    Uint1* sequence; /**< Sequence used for search (could be translation). */
    Uint1* sequence_start; /**< Start of sequence, usually one byte before 
                                sequence as that byte is a NULL sentinel byte.*/
-   Int4     length;         /**< Length of sequence. */
+   Int4 length;         /**< Length of sequence. */
    Int2 frame; /**< Frame of the query, needed for translated searches */
    Int2 subject_strand; /**< Strand of the subject sequence for translated searches. 
                           Uses the same values as ENa_strand. */
@@ -239,6 +249,10 @@ typedef struct BLAST_SequenceBlk {
                                   sequence */
    Boolean sequence_start_allocated; /**< TRUE if memory has been allocated 
                                         for sequence_start */
+   Uint1* sequence_start_nomask; /**< Query sequence without masking. */
+   Uint1* sequence_nomask; /**< Start of query sequence without masking. */
+   Boolean nomask_allocated; /**< If false the two above are just pointers to
+                                   sequence and sequence_start. */
    Uint1* oof_sequence; /**< Mixed-frame protein representation of a
                              nucleotide sequence for out-of-frame alignment */
    Boolean oof_sequence_allocated; /**< TRUE if memory has been allocated 
@@ -264,6 +278,7 @@ typedef struct BLAST_SequenceBlk {
    Uint4 num_seq_ranges;    /**< Number of elements in seq_ranges */
    Boolean seq_ranges_allocated;   /**< TRUE if memory has been allocated for
                                       seq_ranges */
+   Int4 mask_type;          /**< type of subject masking */
    /* END: Data members needed for masking subjects from a BLAST database */
 } BLAST_SequenceBlk;
 
