@@ -1,4 +1,4 @@
-/* $Id: blast_filter.c,v 1.95 2009/05/27 17:39:36 kazimird Exp $
+/* $Id: blast_filter.c,v 1.99 2010/03/31 16:14:46 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,7 +30,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_filter.c,v 1.95 2009/05/27 17:39:36 kazimird Exp $";
+    "$Id: blast_filter.c,v 1.99 2010/03/31 16:14:46 kazimird Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_util.h>
@@ -415,10 +415,19 @@ BlastFilteringOptionsToString(const SBlastFilterOptions* filtering_options)
 
     /* Mask at hash is a modifier for other filtering options, as such it
      * doesn't make sense to apply it by itself */
-    if (SBlastFilterOptionsMaskAtHash(filtering_options) && 
-        strlen(retval) != 0 ) {
-        if (!s_SafeStrCat(&retval, &retval_size, "m;")) {
-            return 0;
+    if (SBlastFilterOptionsMaskAtHash(filtering_options)) {
+        if (strlen(retval) != 0) {
+            /* Add mask at hash as a modifier for other filtering options */
+            if (!s_SafeStrCat(&retval, &retval_size, "m;")) {
+                return 0;
+            }
+        } else {
+            /* We still need to set "m" in a filter string (WB-391, WB-394) */
+            /* The string below can be modified into "mF" or "mL" or
+               whatever is decided to be the conventional meaning */
+            if (!s_SafeStrCat(&retval, &retval_size, "m;")) {
+                return 0;
+            }
         }
     }
 
@@ -1338,11 +1347,35 @@ BlastSetUp_MaskQuery(BLAST_SequenceBlk* query_blk,
 {
     const Boolean kIsNucl = (program_number == eBlastTypeBlastn);
     Int4 context; /* loop variable. */
+    Int4 total_length = 2; /* Length to copy, adding one for beginning and end. */
+    Boolean has_mask = FALSE; /* Check for whether filter_maskloc is empty. */
+    Int4 index; /* loop variable. */
 
     ASSERT(query_blk);
     ASSERT(query_info);
     ASSERT(filter_maskloc);
 
+    
+    for (index=0; index<filter_maskloc->total_size; index++)
+    {
+         if (filter_maskloc->seqloc_array[index])
+         {
+            has_mask = TRUE;
+            break;
+         }
+    }
+    if (has_mask == FALSE)
+       return;
+
+
+    for (context = query_info->first_context;
+         context <= query_info->last_context; ++context) {
+        total_length += query_info->contexts[context].query_length;
+    }
+    query_blk->sequence_start_nomask = BlastMemDup(query_blk->sequence_start, total_length);
+    query_blk->sequence_nomask = query_blk->sequence_start_nomask + 1;
+    query_blk->nomask_allocated = TRUE;
+
     for (context = query_info->first_context;
          context <= query_info->last_context; ++context) {
       
