@@ -64,7 +64,9 @@ static GbFeatName STATIC__ParFlat_GBQual_names[ParFlat_TOTAL_GBQUAL] = {
  {"bio_material", Class_text}, { "ncRNA_class", Class_text},
  {"tag_peptide", Class_text}, { "mating_type", Class_text},
  {"satellite", Class_text}, { "gene_synonym", Class_text},
- { "UniProtKB_evidence", Class_text}
+ {"UniProtKB_evidence", Class_text}, {"haplogroup", Class_text},
+ {"artificial_location", Class_none}, {"non_functional", Class_none},
+ {"pseudogene", Class_none}
 };
 
 NLM_EXTERN GbFeatNamePtr x_ParFlat_GBQual_names(void) {
@@ -133,7 +135,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"C_region", 0, {-1, -1, -1, -1, -1}, 18,
+   {"C_region", 0, {-1, -1, -1, -1, -1}, 20,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -146,18 +148,20 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1}},
+      -1, -1, -1, -1, -1}},
    {"CAAT_signal", 0, {-1, -1, -1, -1, -1}, 15,
      {
       GBQUAL_allele,
@@ -180,9 +184,10 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"CDS", 0, {-1, -1, -1, -1, -1}, 32,
+   {"CDS", 0, {-1, -1, -1, -1, -1}, 35,
      {
       GBQUAL_allele,
+      GBQUAL_artificial_location,
       GBQUAL_citation,
       GBQUAL_codon_start,
       GBQUAL_codon,
@@ -199,6 +204,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_number,
       GBQUAL_old_locus_tag,
@@ -207,6 +213,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_product,
       GBQUAL_protein_id,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_ribosomal_slippage,
       GBQUAL_standard_name,
       GBQUAL_trans_splicing,
@@ -216,8 +223,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1}},
+      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}},
    {"conflict",  1, {
       GBQUAL_citation, -1, -1, -1, -1}, 14,
      {
@@ -263,7 +269,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"D_segment", 0, {-1, -1, -1, -1, -1}, 18,
+   {"D_segment", 0, {-1, -1, -1, -1, -1}, 20,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -276,18 +282,20 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1}},
+      -1, -1, -1, -1, -1}},
    {"enhancer",  0, {-1, -1, -1, -1, -1}, 17,
      {
       GBQUAL_allele,
@@ -312,7 +320,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"exon", 0, {-1, -1, -1, -1, -1}, 21,
+   {"exon", 0, {-1, -1, -1, -1, -1}, 23,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -327,19 +335,21 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_number,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1}},
+      -1, -1}},
    {"gap",  1, {
       GBQUAL_estimated_length, -1, -1, -1, -1}, 5,
      {
@@ -376,7 +386,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"gene", 0, {-1, -1, -1, -1, -1}, 21,
+   {"gene", 0, {-1, -1, -1, -1, -1}, 23,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -390,6 +400,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_operon,
@@ -397,13 +408,14 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_phenotype,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_trans_splicing,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1}},
+      -1, -1}},
     {"iDNA", 0, {-1, -1, -1, -1, -1}, 18,
      {
       GBQUAL_allele,
@@ -429,7 +441,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1}},
-   {"intron", 0, {-1, -1, -1, -1, -1}, 20,
+   {"intron", 0, {-1, -1, -1, -1, -1}, 22,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -444,19 +456,21 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_number,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1}},
-   {"J_segment", 0, {-1, -1, -1, -1, -1}, 18,
+      -1, -1, -1}},
+   {"J_segment", 0, {-1, -1, -1, -1, -1}, 20,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -469,18 +483,20 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1}},
+      -1, -1, -1, -1, -1}},
    {"LTR", 0, {-1, -1, -1, -1, -1}, 16,
      {
       GBQUAL_allele,
@@ -504,7 +520,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"mat_peptide",  0, {-1, -1, -1, -1, -1}, 23,
+   {"mat_peptide",  0, {-1, -1, -1, -1, -1}, 25,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -522,18 +538,19 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1}},
+      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}},
    {"misc_binding",  1, {
       GBQUAL_bound_moiety, -1, -1, -1, -1}, 16,
      {
@@ -585,7 +602,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1}},
-   {"misc_feature",  0, {-1, -1, -1, -1, -1}, 21,
+   {"misc_feature",  0, {-1, -1, -1, -1, -1}, 23,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -599,6 +616,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_number,
       GBQUAL_old_locus_tag,
@@ -606,13 +624,14 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_phenotype,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1}},
+      -1, -1}},
    {"misc_recomb", 0, {-1, -1, -1, -1, -1}, 16,
      {
       GBQUAL_allele,
@@ -636,7 +655,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"misc_RNA",  0, {-1, -1, -1, -1, -1}, 21,
+   {"misc_RNA",  0, {-1, -1, -1, -1, -1}, 23,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -650,12 +669,14 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_operon,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_trans_splicing,
       GBQUAL_usedin,
@@ -663,7 +684,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1}},
+      -1, -1}},
    {"misc_signal",  0, {-1, -1, -1, -1, -1}, 19,
      {
       GBQUAL_allele,
@@ -737,9 +758,10 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"mRNA",  0, {-1, -1, -1, -1, -1}, 22,
+   {"mRNA",  0, {-1, -1, -1, -1, -1}, 25,
      {
       GBQUAL_allele,
+      GBQUAL_artificial_location,
       GBQUAL_citation,
       GBQUAL_db_xref,
       GBQUAL_evidence,
@@ -751,12 +773,14 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_operon,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_trans_splicing,
       GBQUAL_transcript_id,
@@ -764,8 +788,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1}},
+      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}},
    {"mutation",  0, {-1, -1, -1, -1, -1}, 18,
      {
       GBQUAL_citation,
@@ -791,7 +814,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1}},
-   {"N_region",  0, {-1, -1, -1, -1, -1}, 16,
+   {"N_region",  0, {-1, -1, -1, -1, -1}, 18,
      {
       GBQUAL_citation,
       GBQUAL_db_xref,
@@ -803,19 +826,21 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1, -1, -1}},
+      -1, -1, -1, -1, -1, -1, -1}},
    {"ncRNA",  1, {
-      GBQUAL_ncRNA_class, -1, -1, -1, -1}, 21,
+      GBQUAL_ncRNA_class, -1, -1, -1, -1}, 23,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -829,12 +854,14 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_operon,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_trans_splicing,
       GBQUAL_usedin,
@@ -842,7 +869,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1}},
+      -1, -1}},
    {"old_sequence",   1, {
       GBQUAL_citation, -1, -1, -1, -1}, 15,
      {
@@ -867,7 +894,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}},
    {"operon", 1, {
-      GBQUAL_operon, -1, -1, -1, -1}, 14,
+      GBQUAL_operon, -1, -1, -1, -1}, 16,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -878,17 +905,18 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_inference,
       GBQUAL_label,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_partial,
       GBQUAL_phenotype,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1}},
+      -1, -1, -1, -1, -1, -1, -1, -1, -1}},
    {"oriT", 0, {-1, -1, -1, -1, -1}, 23,
      {
       GBQUAL_allele,
@@ -1039,7 +1067,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"promoter",  0, {-1, -1, -1, -1, -1}, 21,
+   {"promoter",  0, {-1, -1, -1, -1, -1}, 23,
      {
       GBQUAL_allele,
       GBQUAL_bound_moiety,
@@ -1054,19 +1082,21 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_operon,
       GBQUAL_partial,
       GBQUAL_phenotype,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1}},
+      -1, -1}},
    {"protein_bind",  1, {
       GBQUAL_bound_moiety, -1, -1, -1, -1}, 18,
      {
@@ -1200,7 +1230,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"rRNA",  0, {-1, -1, -1, -1, -1}, 20,
+   {"rRNA",  0, {-1, -1, -1, -1, -1}, 22,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1214,20 +1244,22 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_operon,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1}},
-   {"S_region",  0, {-1, -1, -1, -1, -1}, 18,
+      -1, -1, -1}},
+   {"S_region",  0, {-1, -1, -1, -1, -1}, 20,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1240,18 +1272,20 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1}},
+      -1, -1, -1, -1, -1}},
    {"satellite",  0, {-1, -1, -1, -1, -1}, 21,
      {
       GBQUAL_allele,
@@ -1280,7 +1314,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1}},
-   {"scRNA",  0, {-1, -1, -1, -1, -1}, 19,
+   {"scRNA",  0, {-1, -1, -1, -1, -1}, 21,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1294,19 +1328,21 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1}},
-   {"sig_peptide", 0, {-1, -1, -1, -1, -1}, 19,
+      -1, -1, -1, -1}},
+   {"sig_peptide", 0, {-1, -1, -1, -1, -1}, 21,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1320,19 +1356,21 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1}},
-   {"snoRNA",  0, {-1, -1, -1, -1, -1}, 19,
+      -1, -1, -1, -1}},
+   {"snoRNA",  0, {-1, -1, -1, -1, -1}, 21,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1346,19 +1384,21 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1}},
-   {"snRNA",  0, {-1, -1, -1, -1, -1}, 19,
+      -1, -1, -1, -1}},
+   {"snRNA",  0, {-1, -1, -1, -1, -1}, 21,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1372,20 +1412,22 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1}},
+      -1, -1, -1, -1}},
    {"source", 1, {
-      GBQUAL_organism, -1, -1, -1, -1}, 62,
+      GBQUAL_organism, -1, -1, -1, -1}, 63,
      {
       GBQUAL_bio_material,
       GBQUAL_cell_line,
@@ -1449,7 +1491,8 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_usedin,
       GBQUAL_variety,
       GBQUAL_virion,
-      -1, -1, -1}},
+      GBQUAL_haplogroup,
+      -1, -1}},
    {"stem_loop",  0, {-1, -1, -1, -1, -1}, 18,
      {
       GBQUAL_allele,
@@ -1544,7 +1587,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"tmRNA",  0, {-1, -1, -1, -1, -1}, 21,
+   {"tmRNA",  0, {-1, -1, -1, -1, -1}, 23,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1558,12 +1601,14 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_operon,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_tag_peptide,
       GBQUAL_usedin,
@@ -1571,8 +1616,8 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1}},
-   {"transit_peptide",  0, {-1, -1, -1, -1, -1}, 19,
+      -1, -1}},
+   {"transit_peptide",  0, {-1, -1, -1, -1, -1}, 21,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1586,19 +1631,21 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1}},
-   {"tRNA",  0, {-1, -1, -1, -1, -1}, 21,
+      -1, -1, -1, -1}},
+   {"tRNA",  0, {-1, -1, -1, -1, -1}, 24,
      {
       GBQUAL_allele,
       GBQUAL_anticodon,
@@ -1613,11 +1660,14 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
+      GBQUAL_operon,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_trans_splicing,
       GBQUAL_usedin,
@@ -1625,7 +1675,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1}},
+      -1}},
    {"unsure",  0, {-1, -1, -1, -1, -1}, 15,
      {
       GBQUAL_allele,
@@ -1648,7 +1698,7 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}},
-   {"V_region",  0, {-1, -1, -1, -1, -1}, 18,
+   {"V_region",  0, {-1, -1, -1, -1, -1}, 20,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1661,19 +1711,21 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1}},
-   {"V_segment",  0, {-1, -1, -1, -1, -1}, 18,
+      -1, -1, -1, -1, -1}},
+   {"V_segment",  0, {-1, -1, -1, -1, -1}, 20,
      {
       GBQUAL_allele,
       GBQUAL_citation,
@@ -1686,18 +1738,20 @@ static SematicFeat STATIC__ParFlat_GBFeat[ParFlat_TOTAL_GBFEAT] = {
       GBQUAL_label,
       GBQUAL_locus_tag,
       GBQUAL_map,
+      GBQUAL_non_functional,
       GBQUAL_note,
       GBQUAL_old_locus_tag,
       GBQUAL_partial,
       GBQUAL_product,
       GBQUAL_pseudo,
+      GBQUAL_pseudogene,
       GBQUAL_standard_name,
       GBQUAL_usedin,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-      -1, -1, -1, -1, -1, -1, -1}},
+      -1, -1, -1, -1, -1}},
    {"variation",  0, {-1, -1, -1, -1, -1}, 21,
      {
       GBQUAL_allele,
