@@ -1,4 +1,4 @@
-/* $Id: ncbi_http_connector.c,v 6.96 2009/07/06 18:49:30 kazimird Exp $
+/* $Id: ncbi_http_connector.c,v 6.107 2010/06/08 15:14:44 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,7 @@
 #include "ncbi_ansi_ext.h"
 #include "ncbi_comm.h"
 #include "ncbi_priv.h"
+#include <connect/ncbi_base64.h>
 #include <connect/ncbi_http_connector.h>
 #include <ctype.h>
 #include <errno.h>
@@ -65,13 +66,6 @@ typedef enum {
     eRM_WaitCalled = 2
 } EReadMode;
 
-typedef enum {
-    eRetry_None = 0,
-    eRetry_Redirect,
-    eRetry_Authenticate,
-    eRetry_ProxyAuthenticate
-} ERetry;
-
 
 /* All internal data necessary to perform the (re)connect and I/O
  *
@@ -94,10 +88,12 @@ typedef struct {
     unsigned             error_header:1;  /* only err.HTTP header on SOME dbg*/
     EBCanConnect         can_connect:2;   /* whether more conns permitted    */
     unsigned             read_header:1;   /* whether reading header          */
-    unsigned             auth_sent:1;     /* authenticate sent               */
     unsigned             shut_down:1;     /* whether shut down for write     */
-    unsigned             reserved:10;     /* MBZ                             */
-    unsigned short       failure_count;   /* incr each failure since open    */
+    unsigned             auth_done:1;     /* website authorization sent      */
+    unsigned       proxy_auth_done:1;     /* proxy authorization sent        */
+    unsigned             reserved:6;      /* MBZ                             */
+    unsigned             minor_fault:3;   /* incr each min failure since maj */
+    unsigned short       major_fault;     /* incr each maj failure since open*/
     unsigned short       code;            /* last response code              */
 
     SOCK                 sock;         /* socket;  NULL if not in "READ" mode*/
@@ -121,69 +117,191 @@ static int                   s_MessageIssued = 0;
 static FHTTP_NcbiMessageHook s_MessageHook   = 0;
 
 
-/* Try to fix connection parameters (called for an unconnected connector) */
-static int/*bool*/ s_Adjust(SHttpConnector* uuu,
-                            char**          retry,
-                            EReadMode       read_mode)
+typedef enum {
+    eRetry_None = 0,
+    eRetry_Redirect,
+    eRetry_Authenticate,
+    eRetry_ProxyAuthenticate
+} ERetry;
+
+
+typedef struct {
+    ERetry      mode;
+    const char* data;
+} SRetry;
+
+
+static int/*bool tri-state inverted*/ x_Authorize(SHttpConnector* uuu,
+                                                  const SRetry*   retry)
 {
+    static const char kProxyAuthorization[] = "Proxy-Authorization: Basic ";
+    static const char kAuthorization[]      = "Authorization: Basic ";
+    char buf[80+sizeof(uuu->net_info->user)*4], *s;
+    size_t taglen, userlen, passlen, len, n;
+    const char *tag, *user, *pass;
+
+    assert(retry  &&  retry->data);
+    assert(strncasecmp(retry->data, "basic", strcspn(retry->data," \t")) == 0);
+    switch (retry->mode) {
+    case eRetry_Authenticate:
+        if (uuu->auth_done)
+            return  1/*failed*/;
+        uuu->auth_done = 1/*true*/;
+        if (uuu->net_info->scheme != eURL_Https
+            &&  !(uuu->flags & fHCC_InsecureRedirect)) {
+            return -1/*prohibited*/;
+        }
+        tag    = kAuthorization;
+        taglen = sizeof(kAuthorization) - 1;
+        user   = uuu->net_info->user;
+        pass   = uuu->net_info->pass;
+        break;
+    case eRetry_ProxyAuthenticate:
+        if (uuu->proxy_auth_done)
+            return 1/*failed*/;
+        uuu->proxy_auth_done = 1/*true*/;
+        tag    = kProxyAuthorization;
+        taglen = sizeof(kProxyAuthorization) - 1;
+        user   = uuu->net_info->http_proxy_user;
+        pass   = uuu->net_info->http_proxy_pass;
+        break;
+    default:
+        assert(0);
+        return 1/*failed*/;
+    }
+    assert(tag  &&  user  &&  pass);
+    if (!*user)
+        return 1/*failed*/;
+    userlen = strlen(user);
+    passlen = strlen(pass);
+    s = buf + sizeof(buf) - passlen;
+    if (pass[0] == '['  &&  pass[passlen - 1] == ']') {
+        if (BASE64_Decode(pass + 1, passlen - 2, &len, s, passlen, &n)
+            &&  len == passlen - 2) {
+            len = n;
+        } else
+            len = 0;
+    } else
+        len = 0;
+    s -= userlen;
+    memcpy(--s, user, userlen);
+    s[userlen++] = ':';
+    if (!len) {
+        memcpy(s + userlen, pass, passlen);
+        len  = userlen + passlen;
+    } else
+        len += userlen;
+    /* usable room */
+    userlen = (size_t)(s - buf);
+    assert(userlen > taglen);
+    userlen -= taglen + 1;
+    BASE64_Encode(s, len, &n, buf + taglen, userlen, &passlen, 0);
+    if (len != n  ||  buf[taglen + passlen])
+        return 1/*failed*/;
+    memcpy(buf, tag, taglen);
+    return !ConnNetInfo_OverrideUserHeader(uuu->net_info, buf);
+}
+
+
+/* Try to fix connection parameters (called for an unconnected connector) */
+static EIO_Status s_Adjust(SHttpConnector* uuu,
+                           const SRetry*   retry,
+                           EReadMode       read_mode)
+{
+    EIO_Status status;
+
     assert(!uuu->sock  &&  uuu->can_connect != eCC_None);
 
-    /* we're here because something is going wrong */
-    if (++uuu->failure_count >= uuu->net_info->max_try) {
-        if (*retry)
-            free(*retry);
-        if (read_mode != eRM_DropUnread  &&  uuu->failure_count > 1) {
-            CORE_LOGF_X(1, eLOG_Error,
-                        ("[HTTP]  Too many failed attempts (%d), giving up",
-                         uuu->failure_count));
-        }
-        uuu->can_connect = eCC_None;
-        return 0/*failure*/;
-    }
-
-    /* adjust info before yet another connection attempt */
-    if (*retry) {
-        int fail/*parse*/;
-        assert(**retry);
-        if (**retry != '?') {
-            if (uuu->net_info->req_method == eReqMethod_Get  ||  !uuu->w_len
-                ||  (uuu->flags & fHCC_InsecureRedirect)) {
-                int secure = uuu->net_info->scheme == eURL_Https ? 1 : 0;
-                *uuu->net_info->args = '\0'/*arguments not inherited*/;
-                fail = !ConnNetInfo_ParseURL(uuu->net_info, *retry);
-                if (!fail  &&  secure  &&  uuu->net_info->scheme != eURL_Https
-                    &&  !(uuu->flags & fHCC_InsecureRedirect)) {
+    if (retry  &&  retry->mode  &&  ++uuu->minor_fault < 3) {
+        int fail;
+        /* adjust info before yet another connection attempt */
+        switch (retry->mode) {
+        case eRetry_Redirect:
+            if (retry->data  &&  retry->data[0] != '?') {
+                if (uuu->net_info->req_method == eReqMethod_Get
+                    ||  (uuu->flags & fHCC_InsecureRedirect)
+                    ||  !uuu->w_len) {
+                    int secure = uuu->net_info->scheme == eURL_Https ? 1 : 0;
+                    *uuu->net_info->args = '\0'/*arguments not inherited*/;
+                    fail = !ConnNetInfo_ParseURL(uuu->net_info, retry->data);
+                    if (!fail  &&  secure
+                        &&  uuu->net_info->scheme != eURL_Https
+                        &&  !(uuu->flags & fHCC_InsecureRedirect)) {
+                        fail = -1;
+                    }
+                } else
                     fail = -1;
-                }
+            } else
+                fail = 1;
+            if (fail) {
+                CORE_LOGF_X(2, eLOG_Error,
+                            ("[HTTP]  %s redirect to %s%s%s",
+                             fail < 0 ? "Prohibited" : "Cannot",
+                             retry->data ? "\""        : "<",
+                             retry->data ? retry->data : "NULL",
+                             retry->data ? "\""        : ">"));
+                status = eIO_Closed;
+            } else
+                status = eIO_Success;
+            break;
+        case eRetry_Authenticate:
+        case eRetry_ProxyAuthenticate:
+            if (retry->data  &&  strncasecmp(retry->data, "basic ", 6) == 0) {
+                fail = x_Authorize(uuu, retry);
+                if (fail < 0)
+                    fail = 2;
             } else
                 fail = -1;
-        } else
-            fail = 1;
-        if (fail) {
-            CORE_LOGF_X(2, eLOG_Error,
-                        ("[HTTP]  %s redirect to \"%s\"",
-                         fail < 0 ? "Prohibited" : "Cannot", *retry));
+            if (fail) {
+                CORE_LOGF_X(3, eLOG_Error,
+                            ("[HTTP]  %s %s %c%s%c",
+                             retry->mode == eRetry_Authenticate
+                             ? "Authorization" : "Proxy authorization",
+                             fail < 0 ? "not yet implemented" :
+                             fail > 1 ? "prohibited" : "failed",
+                             "(<"[!retry->data],
+                             retry->data ? retry->data : "NULL",
+                             ")>"[!retry->data]));
+                status = fail < 0 ? eIO_NotSupported : eIO_Closed;
+            } else
+                status = eIO_Success;
+            break;
+        default:
+            status = eIO_Unknown;
+            assert(0);
+            break;
         }
-        free(*retry);
-        if (fail) {
+        if (status != eIO_Success) {
             uuu->can_connect = eCC_None;
-            return 0/*failure*/;
+            return status;
         }
-    } else if (!uuu->adjust_net_info
-               ||  uuu->adjust_net_info(uuu->net_info,
-                                        uuu->adjust_data,
-                                        uuu->failure_count) == 0) {
-        if (read_mode != eRM_DropUnread  &&  uuu->failure_count > 1) {
-            CORE_LOGF_X(3, eLOG_Error,
-                        ("[HTTP]  Retry attempts (%d) exhausted, giving up",
-                         uuu->failure_count));
+    } else {
+        const char* msg;
+        uuu->minor_fault = 0;
+        /* we're here because something is going wrong */
+        if (++uuu->major_fault >= uuu->net_info->max_try) {
+            msg = read_mode != eRM_DropUnread  &&  uuu->major_fault > 1
+                ? "[HTTP]  Too many failed attempts (%d), giving up" : "";
+        } else if (!uuu->adjust_net_info
+                   ||  uuu->adjust_net_info(uuu->net_info,
+                                            uuu->adjust_data,
+                                            uuu->major_fault) == 0) {
+            msg = read_mode != eRM_DropUnread  &&  uuu->major_fault > 1
+                ? "[HTTP]  Retry attempts (%d) exhausted, giving up" : "";
+        } else
+            msg = 0;
+        if (msg) {
+            if (*msg) {
+                CORE_LOGF_X(1, eLOG_Error,
+                            (msg, uuu->major_fault));
+            }
+            uuu->can_connect = eCC_None;
+            return eIO_Closed;
         }
-        uuu->can_connect = eCC_None;
-        return 0/*failure*/;
     }
 
-    ConnNetInfo_AdjustForHttpProxy(uuu->net_info);
-    return 1/*success*/;
+    return ConnNetInfo_AdjustForHttpProxy(uuu->net_info)
+        ? eIO_Success : eIO_Unknown;
 }
 
 
@@ -192,7 +310,10 @@ static void s_DropConnection(SHttpConnector* uuu, const STimeout* timeout)
 {
     assert(uuu->sock);
     BUF_Erase(uuu->http);
-    SOCK_SetTimeout(uuu->sock, eIO_Close, timeout);
+    if (uuu->read_header)
+        SOCK_Abort(uuu->sock);
+    else
+        SOCK_SetTimeout(uuu->sock, eIO_Close, timeout);
     SOCK_Close(uuu->sock);
     uuu->sock = 0;
 }
@@ -210,7 +331,8 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
 
     assert(!uuu->sock);
     if (uuu->can_connect == eCC_None) {
-        CORE_LOG_X(5, eLOG_Error, "[HTTP]  Connector no longer usable");
+        if (read_mode == eRM_Regular)
+            CORE_LOG_X(5, eLOG_Error, "[HTTP]  Connector no longer usable");
         return eIO_Closed;
     }
 
@@ -218,7 +340,6 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
     for (;;) {
         int/*bool*/ reset_user_header = 0;
         char*       http_user_header = 0;
-        char*       null = 0;
         TSOCK_Flags flags;
 
         uuu->w_len = BUF_Size(uuu->w_buf);
@@ -262,8 +383,8 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
             break/*success*/;
 
         /* connection failed, no socket was created */
-        if (!s_Adjust(uuu, &null, read_mode))
-            break/*closed*/;
+        if (s_Adjust(uuu, 0, read_mode) != eIO_Success)
+            break;
     }
 
     return status;
@@ -283,7 +404,6 @@ static EIO_Status s_ConnectAndSend(SHttpConnector* uuu,
     for (;;) {
         size_t body_size;
         char   buf[4096];
-        char*  null = 0;
 
         if (!uuu->sock) {
             if ((status = s_Connect(uuu, read_mode)) != eIO_Success)
@@ -344,25 +464,73 @@ static EIO_Status s_ConnectAndSend(SHttpConnector* uuu,
                     ("[HTTP]  Error %s (%s)", buf, IO_StatusStr(status)));
 
         /* write failed; close and try to use another server */
-        SOCK_Abort(uuu->sock);
         s_DropConnection(uuu, 0/*no wait*/);
-        if (!s_Adjust(uuu, &null, read_mode)) {
-            uuu->can_connect = eCC_None;
-            status = eIO_Closed;
-            break;
-        }
+        if ((status = s_Adjust(uuu, 0, read_mode)) != eIO_Success)
+            break/*closed*/;
     }
 
     return status;
 }
 
 
+static int/*bool*/ s_IsValidParam(const char* param, size_t parlen)
+{
+    const char* e = (const char*) memchr(param, '=', parlen);
+    size_t toklen;
+    if (!e  ||  e == param)
+        return 0/*false*/;
+    if ((toklen = (size_t)(++e - param)) >= parlen)
+        return 0/*false*/;
+    assert(!isspace((unsigned char)(*param)));
+    if (strcspn(param, " \t") < toklen)
+        return 0/*false*/;
+    if (*e == '\''  ||  *e == '"') {
+        /* a quoted string */
+        toklen = parlen - toklen;
+        if (!(e = (const char*) memchr(e + 1, *e, --toklen)))
+            return 0/*false*/;
+        e++/*skip the quote*/;
+    } else
+        e += strcspn(e, " \t");
+    if (e != param + parlen  &&  e + strspn(e, " \t") != param + parlen)
+        return 0/*false*/;
+    return 1/*true*/;
+}
+
+
+static int/*bool*/ s_IsValidAuth(const char* challenge, size_t len)
+{
+    /* Challenge must contain a scheme name token and a non-empty param
+     * list (comma-separated pairs token={token|quoted_string}), with at
+     * least one parameter being named "realm=".
+     */
+    size_t word = strcspn(challenge, " \t");
+    int retval = 0/*false*/;
+    if (word < len) {
+        /* 1st word is always the scheme name */
+        const char* param = challenge + word;
+        for (param += strspn(param, " \t");  param < challenge + len;
+             param += strspn(param, ", \t")) {
+            size_t parlen = (size_t)(challenge + len - param);
+            const char* c = (const char*) memchr(param, ',', parlen);
+            if (c)
+                parlen = (size_t)(c - param);
+            if (!s_IsValidParam(param, parlen))
+                return 0/*false*/;
+            if (parlen > 6  &&  strncasecmp(param, "realm=", 6) == 0)
+                retval = 1/*true, but keep scanning*/;
+            param += c ? ++parlen : parlen;
+        }
+    }
+    return retval;
+}
+
+
 /* Parse HTTP header */
 static EIO_Status s_ReadHeader(SHttpConnector* uuu,
-                               char**          retry,
+                               SRetry*         retry,
                                EReadMode       read_mode)
 {
-    ERetry     redirect = eRetry_None; 
     int        server_error = 0;
     int        http_status;
     EIO_Status status;
@@ -370,7 +538,8 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
     size_t     size;
 
     assert(uuu->sock  &&  uuu->read_header);
-    *retry = 0;
+    retry->mode = eRetry_None;
+    retry->data = 0;
 
     /* line by line HTTP header input */
     for (;;) {
@@ -418,12 +587,12 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
         http_status = -1;
     if (http_status < 200  ||  299 < http_status) {
         server_error = http_status;
-        if      (http_status == 301  ||  http_status == 302)
-            redirect = eRetry_Redirect;
+        if (http_status == 301  ||  http_status == 302  ||  http_status == 307)
+            retry->mode = eRetry_Redirect;
         else if (http_status == 401)
-            redirect = eRetry_Authenticate;
+            retry->mode = eRetry_Authenticate;
         else if (http_status == 407)
-            redirect = eRetry_ProxyAuthenticate;
+            retry->mode = eRetry_ProxyAuthenticate;
         else if (http_status < 0  ||  http_status == 403 || http_status == 404)
             uuu->net_info->max_try = 0;
     }
@@ -438,7 +607,7 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
             header_header = "HTTP header";
         else if (uuu->flags & fHCC_KeepHeader)
             header_header = "HTTP header (error)";
-        else if (uuu->code == 301  ||  uuu->code == 302)
+        else if (uuu->code == 301  ||  uuu->code == 302  ||  uuu->code == 307)
             header_header = "HTTP header (moved)";
         else if (!uuu->net_info->max_try)
             header_header = "HTTP header (unrecoverable error)";
@@ -495,7 +664,7 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
         server_error = 1/*fake, but still boolean true*/;
     }
 
-    if (redirect == eRetry_Redirect) {
+    if (retry->mode == eRetry_Redirect) {
         /* parsing "Location" pointer */
         static const char kLocationTag[] = "\nLocation: ";
         char* s;
@@ -506,7 +675,8 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
                     location++;
                 if (!(s = strchr(location, '\r')))
                     s = strchr(location, '\n');
-                assert(s);
+                if (!s)
+                    break;
                 do {
                     if (!isspace((unsigned char) s[-1]))
                         break;
@@ -515,25 +685,46 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
                     size_t len = (size_t)(s - location);
                     memmove(header, location, len);
                     header[len] = '\0';
-                    *retry = header;
+                    retry->data = header;
                 }
                 break;
             }
         }
-    } else if (redirect != eRetry_None) {
+    } else if (retry->mode != eRetry_None) {
         /* parsing "Authenticate" tags */
         static const char kAuthenticateTag[] = "-Authenticate: ";
         char* s;
         for (s = strchr(header, '\n');  s  &&  *s;  s = strchr(s + 1, '\n')) {
-            if (strncasecmp(s + (redirect == eRetry_Authenticate ? 4 : 6),
+            if (strncasecmp(s + (retry->mode == eRetry_Authenticate ? 4 : 6),
                             kAuthenticateTag, sizeof(kAuthenticateTag)-1)==0){
-                if ((redirect == eRetry_Authenticate
-                     &&  strncasecmp(s, "\nWWW",   4) != 0)  ||
-                    (redirect == eRetry_ProxyAuthenticate
-                     &&  strncasecmp(s, "\nProxy", 6) != 0)) {
-                    continue;
+                if ((retry->mode == eRetry_Authenticate
+                     &&  strncasecmp(s, "\nWWW",   4) == 0)  ||
+                    (retry->mode == eRetry_ProxyAuthenticate
+                     &&  strncasecmp(s, "\nProxy", 6) == 0)) {
+                    char* challenge = s + sizeof(kAuthenticateTag) - 1, *e;
+                    challenge += retry->mode == eRetry_Authenticate ? 4 : 6;
+                    while (*challenge && isspace((unsigned char)(*challenge)))
+                        challenge++;
+                    if (!(e = strchr(challenge, '\r')))
+                        e = strchr(challenge, '\n');
+                    else
+                        s = e;
+                    if (!e)
+                        break;
+                    do {
+                        if (!isspace((unsigned char) e[-1]))
+                            break;
+                    } while (--e > challenge);
+                    if (e != challenge) {
+                        size_t len = (size_t)(e - challenge);
+                        if (s_IsValidAuth(challenge, len)) {
+                            memmove(header, challenge, len);
+                            header[len] = '\0';
+                            retry->data = header;
+                            break;
+                        }
+                    }
                 }
-                /* TODO */
             }
         }
     } else if (!server_error) {
@@ -564,7 +755,7 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
             }
         }
     }
-    if (!*retry)
+    if (!retry->data)
         free(header);
 
     /* skip & printout the content, if server error was flagged */
@@ -612,9 +803,11 @@ static EIO_Status s_PreRead(SHttpConnector* uuu,
                             EReadMode       read_mode)
 {
     EIO_Status status;
-    char* retry;
 
     for (;;) {
+        EIO_Status adjust_status;
+        SRetry     retry;
+
         status = s_ConnectAndSend(uuu, read_mode);
         if (!uuu->sock) {
             assert(status != eIO_Success);
@@ -627,29 +820,34 @@ static EIO_Status s_PreRead(SHttpConnector* uuu,
         }
 
         /* set timeout */
-        SOCK_SetTimeout(uuu->sock, eIO_Read, timeout);
+        verify(SOCK_SetTimeout(uuu->sock, eIO_Read, timeout) == eIO_Success);
 
         if (!uuu->read_header)
             break;
 
         if ((status = s_ReadHeader(uuu, &retry, read_mode)) == eIO_Success) {
             /* pending output data no longer needed */
-            assert(!uuu->read_header  &&  !retry);
+            assert(!uuu->read_header  &&  !retry.mode);
             BUF_Erase(uuu->w_buf);
             break;
         }
+        assert(status != eIO_Timeout  ||  !retry.mode);
         /* if polling then bail out with eIO_Timeout */
         if (status == eIO_Timeout
             &&  (read_mode == eRM_WaitCalled
                  ||  (timeout  &&  !(timeout->sec | timeout->usec)))) {
+            assert(!retry.data);
             break;
         }
 
         /* HTTP header read error; disconnect and try to use another server */
-        SOCK_Abort(uuu->sock);
         s_DropConnection(uuu, 0/*no wait*/);
-        if (!s_Adjust(uuu, &retry, read_mode)) {
-            uuu->can_connect = eCC_None;
+        adjust_status = s_Adjust(uuu, retry.mode ? &retry : 0, read_mode);
+        if (retry.data)
+            free((void*) retry.data);
+        if (adjust_status != eIO_Success) {
+            if (adjust_status != eIO_Closed)
+                status = adjust_status;
             break;
         }
     }
@@ -738,34 +936,21 @@ static EIO_Status s_Disconnect(SHttpConnector* uuu,
     if (uuu->sock) /* s_PreRead() might have dropped the connection already */
         s_DropConnection(uuu, timeout);
     if (uuu->can_connect == eCC_Once)
-        uuu->can_connect = eCC_None;
+        uuu->can_connect =  eCC_None;
 
     return status;
 }
 
 
-/* Send the accumulated output data(if any) to server, then close socket.
- * Regardless of the flush, clear both input and output buffer.
- * This function is only called to either re-open or close the connector.
+/* Send the accumulated output data(if any) to server, then close the socket.
+ * Regardless of the flush, clear both input and output buffers.
  */
 static void s_FlushAndDisconnect(SHttpConnector* uuu,
-                                 int/*bool*/     close,
                                  const STimeout* timeout)
 {
-    /* store timeouts for later use */
-    if (timeout) {
-        uuu->oo_timeout = *timeout;
-        uuu->o_timeout  = &uuu->oo_timeout;
-        uuu->ww_timeout = *timeout;
-        uuu->w_timeout  = &uuu->ww_timeout;
-    } else {
-        uuu->o_timeout  = timeout;
-        uuu->w_timeout  = timeout;
-    }
-
-    if (close  &&  uuu->can_connect != eCC_None  &&  !uuu->sock
+    if (uuu->can_connect != eCC_None  &&  !uuu->sock
         &&  ((uuu->flags & fHCC_SureFlush)  ||  BUF_Size(uuu->w_buf))) {
-        /* "WRITE" mode and data (or just flag) pending */
+        /* "WRITE" mode and data (or just flag) is still pending */
         s_PreRead(uuu, timeout, eRM_DropUnread);
     }
     s_Disconnect(uuu, timeout, eRM_DropUnread);
@@ -810,11 +995,6 @@ extern "C" {
     static void        s_Setup      (SMetaConnector *meta,
                                      CONNECTOR connector);
     static void        s_Destroy    (CONNECTOR connector);
-#  ifdef IMPLEMENTED__CONN_WaitAsync
-    static EIO_Status s_VT_WaitAsync(CONNECTOR     connector,
-                                     FConnectorAsyncHandler  func,
-                                     SConnectorAsyncHandler* data);
-#  endif
 #ifdef __cplusplus
 } /* extern "C" */
 #endif /* __cplusplus */
@@ -831,26 +1011,7 @@ static const char* s_VT_GetType
 static char* s_VT_Descr
 (CONNECTOR connector)
 {
-    SHttpConnector* uuu = (SHttpConnector*) connector->handle;
-    size_t len = 8/*"http[s]://"*/ + strlen(uuu->net_info->host) +
-        (uuu->net_info->port ? 6/*:port*/ : 0) +
-        (uuu->net_info->http_proxy_adjusted ? 2 : 0) +
-        strlen(uuu->net_info->path) + 1 +
-        (*uuu->net_info->args ? 2 + strlen(uuu->net_info->args) : 1);
-    char* buf = (char*) malloc(len);
-    if (buf) {
-        len = sprintf(buf, "http%s://%s",
-                      &"s"[uuu->net_info->scheme != eURL_Https],
-                      uuu->net_info->host);
-        if (uuu->net_info->port)
-            len += sprintf(&buf[len], ":%hu", uuu->net_info->port);
-        sprintf(&buf[len], "%s%s%s%s%s", uuu->net_info->http_proxy_adjusted
-                ? "<" : *uuu->net_info->path != '/'
-                ? "/" : "", uuu->net_info->path,
-                &"?"[!*uuu->net_info->args], uuu->net_info->args,
-                ">" + !uuu->net_info->http_proxy_adjusted);
-    }
-    return buf;
+    return ConnNetInfo_URL(((SHttpConnector*) connector->handle)->net_info);
 }
 
 
@@ -861,15 +1022,28 @@ static EIO_Status s_VT_Open
     SHttpConnector* uuu = (SHttpConnector*) connector->handle;
 
     /* NOTE: the real connect will be performed on the first "READ", or
-     * "CLOSE", or on "WAIT" on read -- see in "s_ConnectAndSend()";
-     * we just close underlying socket and prepare to open it later */
-    s_FlushAndDisconnect(uuu, 0/*open*/, timeout);
+     * "CLOSE", or on "WAIT" on read -- see in "s_ConnectAndSend()" */
+
+    assert(!uuu->sock);
+
+    /* store timeouts for later use */
+    if (timeout) {
+        uuu->oo_timeout = *timeout;
+        uuu->o_timeout  = &uuu->oo_timeout;
+        uuu->ww_timeout = *timeout;
+        uuu->w_timeout  = &uuu->ww_timeout;
+    } else {
+        uuu->o_timeout  = timeout;
+        uuu->w_timeout  = timeout;
+    }
 
     /* reset the auto-reconnect feature */
-    uuu->can_connect = uuu->flags & fHCC_AutoReconnect
-        ? eCC_Unlimited : eCC_Once;
-    uuu->failure_count = 0;
-    uuu->auth_sent = 0;
+    uuu->can_connect = (uuu->flags & fHCC_AutoReconnect
+                        ? eCC_Unlimited : eCC_Once);
+    uuu->major_fault     = 0;
+    uuu->minor_fault     = 0;
+    uuu->auth_done       = 0;
+    uuu->proxy_auth_done = 0;
 
     return eIO_Success;
 }
@@ -881,19 +1055,18 @@ static EIO_Status s_VT_Wait
  const STimeout* timeout)
 {
     SHttpConnector* uuu = (SHttpConnector*) connector->handle;
+    EIO_Status status;
 
+    assert(event == eIO_Read  ||  event == eIO_Write);
     switch (event) {
     case eIO_Read:
         if (BUF_Size(uuu->r_buf))
             return eIO_Success;
         if (uuu->can_connect == eCC_None)
             return eIO_Closed;
-        if (!uuu->sock  ||  uuu->read_header) {
-            EIO_Status status = s_PreRead(uuu, timeout, eRM_WaitCalled);
-            if (status != eIO_Success)
-                return status;
-            assert(uuu->sock);
-        }
+        if ((status = s_PreRead(uuu, timeout, eRM_WaitCalled)) != eIO_Success)
+            return status;
+        assert(uuu->sock);
         return SOCK_Wait(uuu->sock, eIO_Read, timeout);
     case eIO_Write:
         /* Return 'Closed' if no more writes are allowed (and now - reading) */
@@ -901,9 +1074,9 @@ static EIO_Status s_VT_Wait
             ||  (uuu->sock  &&  uuu->can_connect == eCC_Once)
             ? eIO_Closed : eIO_Success;
     default:
+        assert(0);
         break;
     }
-    assert(0);
     return eIO_InvalidArg;
 }
 
@@ -930,15 +1103,14 @@ static EIO_Status s_VT_Write
         return eIO_Closed; /* no more connects permitted */
 
     /* accumulate all output in the memory buffer */
-    if (uuu->flags & fHCC_UrlEncodeOutput) {
+    if (size  &&  (uuu->flags & fHCC_UrlEncodeOutput)) {
         /* with URL-encoding */
         size_t dst_size = 3 * size;
-        void* dst = malloc(dst_size);
-        size_t dst_written;
-        URL_Encode(buf, size, n_written, dst, dst_size, &dst_written);
-        assert(*n_written == size);
-        if (!BUF_Write(&uuu->w_buf, dst, dst_written)) {
-            free(dst);
+        void*  dst = malloc(dst_size);
+        URL_Encode(buf, size, n_written, dst, dst_size, &dst_size);
+        if (*n_written != size  ||  !BUF_Write(&uuu->w_buf, dst, dst_size)) {
+            if (dst)
+                free(dst);
             return eIO_Unknown;
         }
         free(dst);
@@ -955,7 +1127,6 @@ static EIO_Status s_VT_Write
         uuu->w_timeout  = &uuu->ww_timeout;
     } else
         uuu->w_timeout  = timeout;
-
     return eIO_Success;
 }
 
@@ -1025,40 +1196,25 @@ static EIO_Status s_VT_Close
 (CONNECTOR       connector,
  const STimeout* timeout)
 {
-    s_FlushAndDisconnect((SHttpConnector*) connector->handle,
-                         1/*close*/, timeout);
+    s_FlushAndDisconnect((SHttpConnector*) connector->handle, timeout);
     return eIO_Success;
 }
 
 
-#ifdef IMPLEMENTED__CONN_WaitAsync
-static EIO_Status s_VT_WaitAsync
-(void*                   connector,
- FConnectorAsyncHandler  func,
- SConnectorAsyncHandler* data)
-{
-    return eIO_NotSupported;
-}
-#endif
-
-
 static void s_Setup(SMetaConnector *meta, CONNECTOR connector)
 {
     SHttpConnector* uuu = (SHttpConnector*) connector->handle;
 
     /* initialize virtual table */
-    CONN_SET_METHOD(meta, get_type,   s_VT_GetType,   connector);
-    CONN_SET_METHOD(meta, descr,      s_VT_Descr,     connector);
-    CONN_SET_METHOD(meta, open,       s_VT_Open,      connector);
-    CONN_SET_METHOD(meta, wait,       s_VT_Wait,      connector);
-    CONN_SET_METHOD(meta, write,      s_VT_Write,     connector);
-    CONN_SET_METHOD(meta, flush,      s_VT_Flush,     connector);
-    CONN_SET_METHOD(meta, read,       s_VT_Read,      connector);
-    CONN_SET_METHOD(meta, status,     s_VT_Status,    connector);
-    CONN_SET_METHOD(meta, close,      s_VT_Close,     connector);
-#ifdef IMPLEMENTED__CONN_WaitAsync
-    CONN_SET_METHOD(meta, wait_async, s_VT_WaitAsync, connector);
-#endif
+    CONN_SET_METHOD(meta, get_type, s_VT_GetType, connector);
+    CONN_SET_METHOD(meta, descr,    s_VT_Descr,   connector);
+    CONN_SET_METHOD(meta, open,     s_VT_Open,    connector);
+    CONN_SET_METHOD(meta, wait,     s_VT_Wait,    connector);
+    CONN_SET_METHOD(meta, write,    s_VT_Write,   connector);
+    CONN_SET_METHOD(meta, flush,    s_VT_Flush,   connector);
+    CONN_SET_METHOD(meta, read,     s_VT_Read,    connector);
+    CONN_SET_METHOD(meta, status,   s_VT_Status,  connector);
+    CONN_SET_METHOD(meta, close,    s_VT_Close,   connector);
     CONN_SET_DEFAULT_TIMEOUT(meta, uuu->net_info->timeout);
 }
 
@@ -1066,6 +1222,7 @@ static void s_Setup(SMetaConnector *meta, CONNECTOR connector)
 static void s_Destroy(CONNECTOR connector)
 {
     SHttpConnector* uuu = (SHttpConnector*) connector->handle;
+    connector->handle = 0;
 
     if (uuu->adjust_cleanup)
         uuu->adjust_cleanup(uuu->adjust_data);
@@ -1074,29 +1231,46 @@ static void s_Destroy(CONNECTOR connector)
     BUF_Destroy(uuu->r_buf);
     BUF_Destroy(uuu->w_buf);
     free(uuu);
-    connector->handle = 0;
     free(connector);
 }
 
 
-/* NB: HTTP tag name misspelled as "Referer" as per the standard. */
-static void s_AddReferer(SConnNetInfo* net_info)
+/* NB: per the standard, HTTP tag name misspelled as "Referer" */
+static void s_AddRefererStripCAF(SConnNetInfo* net_info)
 {
     const char* s;
     char* referer;
-    if (!net_info  ||  !net_info->http_referer  ||  !*net_info->http_referer)
+    if (!net_info)
         return;
     if ((s = net_info->http_user_header) != 0) {
-        int/*bool*/ first;
-        for (first = 1;  *s;  first = 0) {
-            if (strncasecmp(s++, "\nReferer: " + first, 10 - first) == 0)
-                return;
-            if (!(s = strchr(s, '\n')))
+        int/*bool*/ found = 0/*false*/;
+        int/*bool*/ first = 1/*true*/;
+        while (*s) {
+            if (strncasecmp(s, "\nReferer: " + first, 10 - first) == 0) {
+                found = 1/*true*/;
+#ifdef HAVE_LIBCONNEXT
+            } else if (strncasecmp(s, "\nCAF" + first, 4 - first) == 0
+                       &&  (s[4 - first] == '-'  ||  s[4 - first] == ':')) {
+                size_t off = (size_t)(s - net_info->http_user_header);
+                ConnNetInfo_DeleteUserHeader(net_info, s + !first);
+                if (!(s = net_info->http_user_header)  ||  !*(s += off))
+                    break;
+                continue;
+#else
                 break;
+#endif /*HAVE_LIBCONNEXT*/
+            }
+            if (!(s = strchr(++s, '\n')))
+                break;
+            first = 0/*false*/;
         }
+        if (found)
+            return;
     }
-    if (!(referer = (char*) malloc(strlen(net_info->http_referer) + 12)))
+    if (!net_info->http_referer  ||  !*net_info->http_referer  ||
+        !(referer = (char*) malloc(strlen(net_info->http_referer) + 12))) {
         return;
+    }
     sprintf(referer, "Referer: %s\r\n", net_info->http_referer);
     ConnNetInfo_ExtendUserHeader(net_info, referer);
     free(referer);
@@ -1116,6 +1290,7 @@ static CONNECTOR s_CreateConnector
     CONNECTOR       ccc;
     SHttpConnector* uuu;
     SConnNetInfo*   xxx;
+    char*           fff;
 
     xxx = net_info ? ConnNetInfo_Clone(net_info) : ConnNetInfo_Create(0);
     if (!xxx)
@@ -1123,13 +1298,17 @@ static CONNECTOR s_CreateConnector
     if ((flags & fHCC_NoAutoRetry)  ||  !xxx->max_try)
         xxx->max_try = 1;
 
+    if ((fff = strchr(xxx->args, '#')) != 0)
+        *fff = '\0';
     if ((xxx->scheme != eURL_Unspec  && 
          xxx->scheme != eURL_Https   &&
-         xxx->scheme != eURL_Http)   ||
-        !ConnNetInfo_AdjustForHttpProxy(xxx)) {
+         xxx->scheme != eURL_Http)
+        ||  !ConnNetInfo_AdjustForHttpProxy(xxx)) {
         ConnNetInfo_Destroy(xxx);
         return 0;
     }
+    if (xxx->scheme == eURL_Unspec)
+        xxx->scheme = eURL_Http;
 
     if (!(ccc = (SConnector    *) malloc(sizeof(SConnector    )))  ||
         !(uuu = (SHttpConnector*) malloc(sizeof(SHttpConnector)))) {
@@ -1140,40 +1319,33 @@ static CONNECTOR s_CreateConnector
     }
     if (user_header)
         ConnNetInfo_OverrideUserHeader(xxx, user_header);
-    s_AddReferer(xxx);
+    s_AddRefererStripCAF(xxx);
 
     ConnNetInfo_GetValue(0, "HTTP_INSECURE_REDIRECT", value, sizeof(value),"");
-    if (*value  &&  (strcmp    (value, "1")    == 0  ||
-                     strcasecmp(value, "true") == 0  ||
-                     strcasecmp(value, "yes")  == 0  ||
-                     strcasecmp(value, "on")   == 0)) {
+    if (ConnNetInfo_Boolean(value))
         flags |= fHCC_InsecureRedirect;
-    }
 
     /* initialize internal data structure */
-    uuu->net_info         = xxx;
+    uuu->net_info        = xxx;
 
-    uuu->parse_http_hdr   = parse_http_hdr;
-    uuu->adjust_net_info  = adjust_net_info;
-    uuu->adjust_cleanup   = adjust_cleanup;
-    uuu->adjust_data      = adjust_data;
+    uuu->parse_http_hdr  = parse_http_hdr;
+    uuu->adjust_net_info = adjust_net_info;
+    uuu->adjust_cleanup  = adjust_cleanup;
+    uuu->adjust_data     = adjust_data;
 
-    uuu->flags            = flags;
-    uuu->reserved         = 0;
-    uuu->can_connect      = eCC_Once;         /* will be properly set at open*/
+    uuu->flags           = flags;
+    uuu->reserved        = 0;
+    uuu->can_connect     = eCC_None;         /* will be properly set at open*/
 
     ConnNetInfo_GetValue(0, "HTTP_ERROR_HEADER_ONLY", value, sizeof(value),"");
-    uuu->error_header = (*value  &&  (strcmp    (value, "1")    == 0  ||
-                                      strcasecmp(value, "true") == 0  ||
-                                      strcasecmp(value, "yes")  == 0  ||
-                                      strcasecmp(value, "on")   == 0));
+    uuu->error_header    = ConnNetInfo_Boolean(value);
 
-    uuu->sock             = 0;
-    uuu->o_timeout        = kDefaultTimeout;  /* deliberately bad values --  */
-    uuu->w_timeout        = kDefaultTimeout;  /* must be reset prior to use  */
-    uuu->http             = 0;
-    uuu->r_buf            = 0;
-    uuu->w_buf            = 0;
+    uuu->sock            = 0;
+    uuu->o_timeout       = kDefaultTimeout;  /* deliberately bad values --  */
+    uuu->w_timeout       = kDefaultTimeout;  /* must be reset prior to use  */
+    uuu->http            = 0;
+    uuu->r_buf           = 0;
+    uuu->w_buf           = 0;
     /* there are some unintialized fields left -- they are initted later */
 
     /* initialize connector structure */
