@@ -1,4 +1,4 @@
-/* $Id: ncbi_sendmail.c,v 6.42 2009/03/26 15:29:29 kazimird Exp $
+/* $Id: ncbi_sendmail.c,v 6.45 2010/04/05 13:59:32 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -188,11 +188,13 @@ static int/*bool*/ s_SockWrite(SOCK sock, const char* buf, size_t len)
 }
 
 
-static void s_MakeFrom(char* buf, size_t size)
+static void s_MakeFrom(char* buf, size_t size, const char* user)
 {
     size_t len;
 
-    if (!CORE_GetUsername(buf, size)  ||  !*buf)
+    if (user  &&  *user)
+        strncpy0(buf, user, size - 1);
+    else if (!CORE_GetUsername(buf, size)  ||  !*buf)
         strncpy0(buf, "anonymous", size - 1);
     len = strlen(buf);
     size -= len;
@@ -201,19 +203,32 @@ static void s_MakeFrom(char* buf, size_t size)
         *buf++ = '@';
         if ((!SOCK_gethostbyaddr(0, buf, size)  ||  !strchr(buf, '.'))
             &&  SOCK_gethostname(buf, size) != 0) {
-            *--buf = '\0';
+            const char* host = getenv("HOSTNAME");
+            if (!host  &&  !(host = getenv("HOST")))
+                *--buf = '\0';
+            else
+                strncpy0(buf, host, size - 1);
         }
     }
 }
 
 
+/* FIXME:  To remove altogether */
+#undef SendMailInfo_Init
 SSendMailInfo* SendMailInfo_Init(SSendMailInfo* info)
+{
+    return SendMailInfo_InitEx(info, 0);
+}
+
+
+SSendMailInfo* SendMailInfo_InitEx(SSendMailInfo* info,
+                                   const char*    user)
 {
     if (info) {
         info->magic_number    = MX_MAGIC_NUMBER;
         info->cc              = 0;
         info->bcc             = 0;
-        s_MakeFrom(info->from, sizeof(info->from));
+        s_MakeFrom(info->from, sizeof(info->from), user);
         info->header          = 0;
         info->body_size       = 0;
         info->mx_host         = "mailgw.ncbi.nlm.nih.gov";
@@ -372,6 +387,7 @@ const char* CORE_SendMailEx(const char*          to,
         SENDMAIL_RETURN(8, "Cannot connect to sendmail");
     }
     SOCK_SetTimeout(sock, eIO_ReadWrite, &info->mx_timeout);
+    SOCK_SetTimeout(sock, eIO_Close,     &info->mx_timeout);
 
     /* Follow the protocol conversation, RFC821 */
     if (!SENDMAIL_READ_RESPONSE(220, 0, buffer))
