@@ -1,4 +1,4 @@
-/* $Id: ncbi_socket_connector.c,v 6.24 2008/10/31 11:25:51 kazimird Exp $
+/* $Id: ncbi_socket_connector.c,v 6.27 2010/05/10 16:59:31 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,6 +34,7 @@
  */
 
 #include "ncbi_ansi_ext.h"
+#include "ncbi_assert.h"
 #include <connect/ncbi_socket_connector.h>
 #include <assert.h>
 #include <stdio.h>
@@ -50,11 +51,11 @@
  */
 typedef struct {
     SOCK           sock;      /* socket;  NULL if not connected yet       */
-    char*          host;      /* server:  host                            */
+    const char*    host;      /* server:  host                            */
     unsigned short port;      /* server:  service port                    */
-    unsigned int   max_try;   /* max.number of attempts to establish conn */
-    void*          init_data; /* data to send to the server on connect    */
-    size_t         init_size; /* size of the "inst_str" buffer            */
+    unsigned int   try_own;   /* max.number of attempts to establish conn */
+    const void*    init_data; /* data to send to the server on connect    */
+    size_t         init_size; /* size of the "init_data" buffer           */
     TSOCK_Flags    flags;     /* see socket flags in ncbi_socket.h        */
 } SSockConnector;
 
@@ -90,11 +91,6 @@ extern "C" {
     static void        s_Setup      (SMetaConnector* meta,
                                      CONNECTOR       connector);
     static void        s_Destroy    (CONNECTOR       connector);
-#  ifdef IMPLEMENTED__CONN_WaitAsync
-    static EIO_Status s_VT_WaitAsync(void*                   connector,
-                                     FConnectorAsyncHandler  func,
-                                     SConnectorAsyncHandler* data);
-#  endif
 #ifdef __cplusplus
 } /* extern "C" */
 #endif /* __cplusplus */
@@ -125,61 +121,40 @@ static EIO_Status s_VT_Open
  const STimeout* timeout)
 {
     SSockConnector* xxx = (SSockConnector*) connector->handle;
-    EIO_Status   status = eIO_Unknown;
-    unsigned int i = 0;
+    EIO_Status status = eIO_Success;
 
-    do {
-        if (xxx->sock  &&  !xxx->host) {
-            /* on-top (connected) connector for the 1st time - only once here*/
-            unsigned int   host;
-            unsigned short port;
-            char           addr[MAX_IP_ADDR_LEN];
-
-            SOCK_GetPeerAddress(xxx->sock, &host, &port, eNH_HostByteOrder);
-            if (SOCK_ntoa(SOCK_HostToNetLong(host), addr, sizeof(addr)) != 0)
-                return eIO_Unknown;
-            xxx->host = strdup(addr);
-            xxx->port = port;
-            status = eIO_Success;
-        } else {
-            if (!xxx->max_try)
+    if (!xxx->sock) {
+        unsigned int i;
+        assert(xxx->try_own);
+        for (i = 0;  i < xxx->try_own;  i++) {
+            /* connect */
+            status = SOCK_CreateEx(xxx->host, xxx->port, timeout, &xxx->sock,
+                                   xxx->init_data, xxx->init_size, xxx->flags);
+            if (xxx->sock) {
+                if (xxx->init_data) {
+                    assert(xxx->init_size);
+                    free((void*) xxx->init_data);
+                    xxx->init_data = 0;
+                }
                 break;
-            /* connect/reconnect */
-            status = xxx->sock ?
-                SOCK_Reconnect(xxx->sock, 0, 0, timeout) :
-                SOCK_CreateEx(xxx->host, xxx->port, timeout, &xxx->sock,
-                              xxx->init_data, xxx->init_size,
-                              xxx->flags);
-            if (xxx->init_data) {
-                free(xxx->init_data);
-                xxx->init_data = 0;
-                xxx->init_size = 0;
             }
-            i++;
+            assert(status != eIO_Success);
         }
-    } while (status != eIO_Success  &&  i < xxx->max_try);
-
+    }
+    assert(xxx->sock  ||  status != eIO_Success);
     return status;
 }
 
 
-static EIO_Status s_VT_Status
-(CONNECTOR connector,
- EIO_Event dir)
-{
-    SSockConnector* xxx = (SSockConnector*) connector->handle;
-    return xxx->sock ? SOCK_Status(xxx->sock, dir) : eIO_Success;
-}
-
-
 static EIO_Status s_VT_Wait
 (CONNECTOR       connector,
  EIO_Event       event,
  const STimeout* timeout)
 {
     SSockConnector* xxx = (SSockConnector*) connector->handle;
-    assert(event == eIO_Read || event == eIO_Write);
-    return xxx->sock ? SOCK_Wait(xxx->sock, event, timeout) : eIO_Closed;
+    assert(event == eIO_Read  ||  event == eIO_Write);
+    assert(xxx->sock);
+    return SOCK_Wait(xxx->sock, event, timeout);
 }
 
 
@@ -191,10 +166,8 @@ static EIO_Status s_VT_Write
  const STimeout* timeout)
 {
     SSockConnector* xxx = (SSockConnector*) connector->handle;
-
-    if (!xxx->sock)
-        return eIO_Closed;
-    SOCK_SetTimeout(xxx->sock, eIO_Write, timeout);
+    assert(xxx->sock);
+    verify(SOCK_SetTimeout(xxx->sock, eIO_Write, timeout) == eIO_Success);
     return SOCK_Write(xxx->sock, buf, size, n_written, eIO_WritePlain);
 }
 
@@ -207,58 +180,53 @@ static EIO_Status s_VT_Read
  const STimeout* timeout)
 {
     SSockConnector* xxx = (SSockConnector*) connector->handle;
-    if (!xxx->sock)
-        return eIO_Closed;
-    SOCK_SetTimeout(xxx->sock, eIO_Read, timeout);
+    assert(xxx->sock);
+    verify(SOCK_SetTimeout(xxx->sock, eIO_Read, timeout) == eIO_Success);
     return SOCK_Read(xxx->sock, buf, size, n_read, eIO_ReadPlain);
 }
 
 
+static EIO_Status s_VT_Status
+(CONNECTOR connector,
+ EIO_Event dir)
+{
+    SSockConnector* xxx = (SSockConnector*) connector->handle;
+    assert(xxx->sock);
+    return SOCK_Status(xxx->sock, dir);
+}
+
+
 static EIO_Status s_VT_Close
 (CONNECTOR       connector,
  const STimeout* timeout)
 {
     SSockConnector* xxx = (SSockConnector*) connector->handle;
     EIO_Status status = eIO_Success;
-
-    if (xxx->sock) {
-        SOCK_SetTimeout(xxx->sock, eIO_Write, timeout);
+    assert(xxx->sock);
+    if (xxx->try_own) {
+        SOCK_SetTimeout(xxx->sock, eIO_Close, timeout);
         status = SOCK_Close(xxx->sock);
-        xxx->sock = 0;
     }
+    xxx->sock = 0;
     return status;
 }
 
 
-#ifdef IMPLEMENTED__CONN_WaitAsync
-static EIO_Status s_VT_WaitAsync
-(void*                   connector,
- FConnectorAsyncHandler  func,
- SConnectorAsyncHandler* data)
-{
-    return eIO_NotSupported;
-}
-#endif
-
-
 static void s_Setup
 (SMetaConnector* meta,
  CONNECTOR       connector)
 {
     /* initialize virtual table */
-    CONN_SET_METHOD(meta, get_type,   s_VT_GetType,   connector);
-    CONN_SET_METHOD(meta, descr,      s_VT_Descr,     connector);
-    CONN_SET_METHOD(meta, open,       s_VT_Open,      connector);
-    CONN_SET_METHOD(meta, wait,       s_VT_Wait,      connector);
-    CONN_SET_METHOD(meta, write,      s_VT_Write,     connector);
-    CONN_SET_METHOD(meta, flush,      0,              0);
-    CONN_SET_METHOD(meta, read,       s_VT_Read,      connector);
-    CONN_SET_METHOD(meta, status,     s_VT_Status,    connector);
-    CONN_SET_METHOD(meta, close,      s_VT_Close,     connector);
-#ifdef IMPLEMENTED__CONN_WaitAsync
-    CONN_SET_METHOD(meta, wait_async, s_VT_WaitAsync, connector);
-#endif
-    meta->default_timeout = 0; /*infinite*/
+    CONN_SET_METHOD(meta, get_type, s_VT_GetType, connector);
+    CONN_SET_METHOD(meta, descr,    s_VT_Descr,   connector);
+    CONN_SET_METHOD(meta, open,     s_VT_Open,    connector);
+    CONN_SET_METHOD(meta, wait,     s_VT_Wait,    connector);
+    CONN_SET_METHOD(meta, write,    s_VT_Write,   connector);
+    CONN_SET_METHOD(meta, flush,    0,            0);
+    CONN_SET_METHOD(meta, read,     s_VT_Read,    connector);
+    CONN_SET_METHOD(meta, status,   s_VT_Status,  connector);
+    CONN_SET_METHOD(meta, close,    s_VT_Close,   connector);
+    meta->default_timeout = kInfiniteTimeout;
 }
 
 
@@ -266,13 +234,15 @@ static void s_Destroy
 (CONNECTOR connector)
 {
     SSockConnector* xxx = (SSockConnector*) connector->handle;
-
-    if (xxx->host)
-        free(xxx->host);
-    if (xxx->init_data)
-        free(xxx->init_data);
-    free(xxx);
     connector->handle = 0;
+
+    xxx->host = 0;
+    if (xxx->init_data) {
+        assert(xxx->init_size);
+        free((void*) xxx->init_data);
+        xxx->init_data = 0;
+    }
+    free(xxx);
     free(connector);
 }
 
@@ -281,32 +251,45 @@ static CONNECTOR s_Init
 (SOCK           sock,
  const char*    host,
  unsigned short port,
- unsigned int   max_try,
+ unsigned int   try_own,
  const void*    init_data,
  size_t         init_size,
  TSOCK_Flags    flags)
 {
     CONNECTOR       ccc = (SConnector    *) malloc(sizeof(SConnector));
-    SSockConnector* xxx = (SSockConnector*) malloc(sizeof(*xxx));
+    SSockConnector* xxx = (SSockConnector*) malloc(sizeof(*xxx) +
+                                                   (port
+                                                    ? strlen(host) + 1
+                                                    : MAX_IP_ADDR_LEN));
 
-    /* parameter check: either sock or host/port, not both */
+    /* sanity check: either sock or host/port, not both */
     assert((!sock && host && port)  ||  (sock && !host && !port));
-    assert(!init_size || init_data);
+    assert(!sock  ||  !(init_size || init_data || flags));  
+    assert(!init_size  ||  init_data);
+
     /* initialize internal data structures */
-    xxx->sock        = sock;
-    xxx->host        = host ? strdup(host) : 0;
-    xxx->port        = port;
-    if (sock)
-        xxx->max_try = SOCK_IsServerSide(sock) ? 0 : max_try;
-    else
-        xxx->max_try = max_try ? max_try : 1;
-    xxx->flags       = flags;
-    xxx->init_size   = init_data ? init_size : 0;
-    if (xxx->init_size) {
-        xxx->init_data = malloc(init_size);
-        memcpy(xxx->init_data, init_data, xxx->init_size);
-    } else {
+    if (sock) {
+        unsigned int x_host;
+        char* addr = (char*) xxx + sizeof(*xxx);
+        SOCK_GetPeerAddress(sock, &x_host, &xxx->port, eNH_HostByteOrder);
+        SOCK_ntoa(SOCK_HostToNetLong(x_host), addr, MAX_IP_ADDR_LEN);
+        xxx->sock      = sock;
+        xxx->host      = addr;
+        assert(xxx->port);
+        xxx->try_own   = try_own ? 1       : 0;
         xxx->init_data = 0;
+    } else {
+        xxx->sock      = 0;
+        xxx->host      = strcpy((char*) xxx + sizeof(*xxx), host);
+        xxx->port      = port;
+        xxx->try_own   = try_own ? try_own : 1;
+        xxx->flags     = flags;
+        xxx->init_size = init_data ? init_size : 0;
+        if (xxx->init_size) {
+            void* data     = malloc(init_size);
+            xxx->init_data = data ? memcpy(data, init_data, init_size) : 0;
+        } else
+            xxx->init_data = 0;
     }
 
     /* initialize connector data */
@@ -329,7 +312,7 @@ extern CONNECTOR SOCK_CreateConnector
  unsigned short port,
  unsigned int   max_try)
 {
-    return s_Init(0, host, port, max_try, 0, 0, fSOCK_LogDefault);
+    return s_Init(0,    host, port, max_try,  0,         0, fSOCK_LogDefault);
 }
 
 
@@ -341,22 +324,13 @@ extern CONNECTOR SOCK_CreateConnectorEx
  size_t         init_size,
  TSOCK_Flags    flags)
 {
-    return s_Init(0, host, port, max_try, init_data, init_size, flags);
+    return s_Init(0,    host, port, max_try,  init_data, init_size, flags);
 }
 
 
 extern CONNECTOR SOCK_CreateConnectorOnTop
-(SOCK         sock,
- unsigned int max_try)
+(SOCK                 sock,
+ unsigned int/*bool*/ own_sock)
 {
-    return s_Init(sock, 0, 0, max_try, 0, 0, fSOCK_LogDefault);
-}
-
-
-extern CONNECTOR SOCK_CreateConnectorOnTopEx
-(SOCK         sock,
- unsigned int max_try,
- TSOCK_Flags  flags)
-{
-    return s_Init(sock, 0, 0, max_try, 0, 0, flags);
+    return s_Init(sock, 0,    0,    own_sock, 0,         0,         0);
 }
