@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_TYPES__H
 #define CONNECT___NCBI_TYPES__H
 
-/*  $Id: ncbi_types.h,v 6.14 2007/08/03 12:55:44 kazimird Exp $
+/* $Id: ncbi_types.h,v 6.16 2009/11/10 15:54:40 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -63,8 +63,10 @@ extern "C" {
 
 
 /** Timeout structure
+ *
+ * @sa CTimeout, g_CTimeoutToSTimeout, g_STimeoutToCTimeout
  */
-typedef struct {
+typedef struct STimeoutTag {
     unsigned int sec;  /**< seconds (truncated to the platf.-dep. max. limit)*/
     unsigned int usec; /**< microseconds (always truncated by mod. 1,000,000)*/
 } STimeout;
@@ -84,6 +86,17 @@ extern NCBI_XCONNECT_EXPORT STimeout*     NcbiMsToTimeout
  );
 
 
+#ifndef NCBI_ESWITCH_DEFINED
+#define NCBI_ESWITCH_DEFINED
+
+/*
+ * ATTENTION!   Do not change this enumeration!
+ *
+ * It must always be kept in sync with its C++ counterpart defined in
+ * "corelib/ncbimisc.hpp". If you absolutely(sic!) need to alter this
+ * type, please apply equivalent changes to both definitions.
+ */
+
 /** Aux. enum to set/unset/default various features
  */
 typedef enum {
@@ -92,6 +105,8 @@ typedef enum {
     eDefault
 } ESwitch;
 
+#endif  /* !defined(NCBI_ESWITCH_DEFINED) */
+
 
 /** Fixed size analogs of size_t and time_t (mainly for IPC)
  */
