@@ -1,4 +1,4 @@
-/* $Id: ncbi_util.c,v 6.59 2009/02/06 16:14:36 kazimird Exp $
+/* $Id: ncbi_util.c,v 6.64 2010/05/29 01:14:46 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -322,7 +322,7 @@ extern const char* NcbiMessagePlusError
         if (*dynamic  &&  message)
             free((void*) message);
         *dynamic = 0;
-        return "<Ouch! Out of memory>";
+        return "Ouch! Out of memory";
     }
 
     if (message) {
@@ -429,7 +429,8 @@ extern char* LOG_ComposeMessage
 
     if (call_data->raw_size) {
         data_len = (sizeof(kRawData_Begin) + 20
-                    + UTIL_PrintableStringSize(call_data->raw_data,
+                    + UTIL_PrintableStringSize((const char*)
+                                               call_data->raw_data,
                                                call_data->raw_size) +
                     sizeof(kRawData_End));
     }
@@ -474,7 +475,8 @@ extern char* LOG_ComposeMessage
                      (unsigned long) call_data->raw_size,
                      &"s"[call_data->raw_size == 1]);
 
-        s = UTIL_PrintableString(call_data->raw_data,
+        s = UTIL_PrintableString((const char*)
+                                 call_data->raw_data,
                                  call_data->raw_size,
                                  s, format_flags & fLOG_FullOctal);
 
@@ -615,7 +617,7 @@ extern const char* CORE_GetUsername(char* buf, size_t bufsize)
     struct passwd* pw;
 #  if !defined(NCBI_OS_SOLARIS)  &&  defined(NCBI_HAVE_GETPWUID_R)
     struct passwd pwd;
-    char pwdbuf[256];
+    char pwdbuf[1024];
 #  endif
 #elif defined(NCBI_OS_MSWIN)
     char  loginbuf[256 + 1];
@@ -670,12 +672,16 @@ extern const char* CORE_GetUsername(char* buf, size_t bufsize)
 #  ifndef NCBI_OS_SOLARIS
     CORE_LOCK_WRITE;
 #  endif
-    if ((pw = getpwuid(getuid())) != 0  &&  pw->pw_name)
-        strncpy0(buf, pw->pw_name, bufsize - 1);
+    if ((pw = getpwuid(getuid())) != 0) {
+        if (pw->pw_name)
+            strncpy0(buf, pw->pw_name, bufsize - 1);
+        else
+            pw = 0;
+    }
 #  ifndef NCBI_OS_SOLARIS
     CORE_UNLOCK;
 #  endif
-    if (pw  &&  pw->pw_name)
+    if (pw)
         return buf;
 #  elif defined(NCBI_HAVE_GETPWUID_R)
 #    if   NCBI_HAVE_GETPWUID_R == 4
@@ -857,26 +863,77 @@ static void s_CRC32_Init(void)
 #endif /*NCBI_USE_PRECOMPILED_CRC32_TABLES*/
 
 
-extern unsigned int CRC32_Update(unsigned int checksum,
-                                 const void *ptr, size_t count)
+extern unsigned int UTIL_CRC32_Update(unsigned int checksum,
+                                      const void *ptr, size_t len)
 {
-    size_t j;
-    const char* str = (const char*) ptr;
+    const unsigned char* data = (const unsigned char*) ptr;
+    size_t i;
 
 #ifndef NCBI_USE_PRECOMPILED_CRC32_TABLES
     s_CRC32_Init();
 #endif /*NCBI_USE_PRECOMPILED_CRC32_TABLES*/
 
-    for (j = 0;  j < count;  j++) {
-        size_t i = ((checksum >> 24) ^ *str++) & 0xFF;
+    for (i = 0;  i < len;  i++) {
+        size_t k = ((checksum >> 24) ^ *data++) & 0xFF;
         checksum <<= 8;
-        checksum  ^= s_CRC32Table[i];
+        checksum  ^= s_CRC32Table[k];
     }
 
     return checksum;
 }
 
 
+#define MOD_ADLER          65521
+#define MAXLEN_ADLER       5548  /* max len to run without overflows */
+#define ADJUST_ADLER(a)    a = (a & 0xFFFF) + (a >> 16) * (0x10000 - MOD_ADLER)
+#define FINALIZE_ADLER(a)  if (a >= MOD_ADLER) a -= MOD_ADLER
+
+unsigned int UTIL_Adler32_Update(unsigned int checksum,
+                                 const void* ptr, size_t len)
+{
+    const unsigned char* data = (const unsigned char*) ptr;
+    unsigned int a = checksum & 0xFFFF, b = checksum >> 16;
+
+    while (len) {
+        size_t i;
+        if (len >= MAXLEN_ADLER) {
+            len -= MAXLEN_ADLER;
+            for (i = 0;  i < MAXLEN_ADLER/4;  ++i) {
+                b += a += data[0];
+                b += a += data[1];
+                b += a += data[2];
+                b += a += data[3];
+                data += 4;
+            }
+        } else {
+            for (i = len >> 2;  i;  --i) {
+                b += a += data[0];
+                b += a += data[1];
+                b += a += data[2];
+                b += a += data[3];
+                data += 4;
+            }
+            for (len &= 3;  len;  --len) {
+                b += a += *data++;
+            }
+        }
+        ADJUST_ADLER(a);
+        ADJUST_ADLER(b);
+    }
+    /* It can be shown that a <= 0x1013A here, so a single subtract will do. */
+    FINALIZE_ADLER(a);
+    /* It can be shown that b can reach 0xFFEF1 here. */
+    ADJUST_ADLER(b);
+    FINALIZE_ADLER(b);
+    return (b << 16) | a;
+}
+
+#undef MOD_ADLER
+#undef MAXLEN_ADLER
+#undef ADJUST_ADLER
+#undef FINALIZE_ADLER
+
+
 
 /******************************************************************************
  *  MISCELLANEOUS
@@ -928,11 +985,11 @@ extern int/*bool*/ UTIL_MatchesMask(const char* name, const char* mask)
 extern char* UTIL_NcbiLocalHostName(char* hostname)
 {
     static const struct {
-        const char* text;
-        size_t      len;
+        const char*  text;
+        const size_t len;
     } kEndings[] = {
-        { ".ncbi.nlm.nih.gov", 17},
-        { ".ncbi.nih.gov", 13}
+        {".ncbi.nlm.nih.gov", 17},
+        {".ncbi.nih.gov",     13}
     };
     size_t len = hostname ? strlen(hostname) : 0;
 
