@@ -1,4 +1,4 @@
-/*  $Id: test_ncbi_service_connector.c,v 6.34 2005/07/15 21:16:03 lavr Exp $
+/* $Id: test_ncbi_service_connector.c,v 6.41 2010/07/12 18:39:31 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,9 +30,9 @@
  *
  */
 
-#include "../ncbi_ansi_ext.h"
-#include "../ncbi_priv.h"
 #include <connect/ncbi_service_connector.h>
+#include "../ncbi_ansi_ext.h"
+#include "../ncbi_priv.h"               /* CORE logging facilities */
 #include <stdlib.h>
 #include <time.h>
 /* This header must go last */
@@ -42,12 +42,10 @@
 int main(int argc, const char* argv[])
 {
     const char* service = argc > 1 && *argv[1] ? argv[1] : "bounce";
-    const char* host = argc > 2 && *argv[2] ? argv[2] : "www.ncbi.nlm.nih.gov";
     static char obuf[8192 + 2];
     SConnNetInfo* net_info;
     CONNECTOR connector;
     EIO_Status status;
-    STimeout* timeout;
     char ibuf[1024];
     CONN conn;
     size_t n;
@@ -55,19 +53,16 @@ int main(int argc, const char* argv[])
     g_NCBI_ConnectRandomSeed = (int) time(0) ^ NCBI_CONNECT_SRAND_ADDEND;
     srand(g_NCBI_ConnectRandomSeed);
 
-    CORE_SetLOGFormatFlags(fLOG_Full | fLOG_DateTime);
+    CORE_SetLOGFormatFlags(fLOG_None          | fLOG_Level   |
+                           fLOG_OmitNoteLevel | fLOG_DateTime);
     CORE_SetLOGFILE(stderr, 0/*false*/);
 
     net_info = ConnNetInfo_Create(service);
-    strcpy(net_info->host, host);
-    if (argc > 3) {
-        strncpy0(obuf, argv[3], sizeof(obuf) - 2);
-        obuf[n = strlen(obuf)] = '\n';
-        obuf[++n]              = 0;
-    } else {
-    }
-    strcpy(net_info->args, "testarg=testval&service=none");
-    timeout = net_info->timeout;
+    ConnNetInfo_AppendArg(net_info, "testarg",  "val");
+    ConnNetInfo_AppendArg(net_info, "service",  "none");
+    ConnNetInfo_AppendArg(net_info, "platform", "none");
+    ConnNetInfo_AppendArg(net_info, "address",  "2010");
+    ConnNetInfo_LogEx(net_info, eLOG_Note, CORE_GetLOG());
 
     connector = SERVICE_CreateConnectorEx(service, fSERV_Any, net_info, 0);
 
@@ -77,7 +72,10 @@ int main(int argc, const char* argv[])
     if (CONN_Create(connector, &conn) != eIO_Success)
         CORE_LOG(eLOG_Fatal, "Failed to create connection");
 
-    if (argc > 3) {
+    if (argc > 2) {
+        strncpy0(obuf, argv[2], sizeof(obuf) - 2);
+        obuf[n = strlen(obuf)] = '\n';
+        obuf[++n]              = '\0';
         if (CONN_Write(conn, obuf, strlen(obuf), &n, eIO_WritePersist)
             != eIO_Success) {
             CONN_Close(conn);
@@ -88,7 +86,7 @@ int main(int argc, const char* argv[])
         for (n = 0; n < 10; n++) {
             size_t m;
             for (m = 0; m < sizeof(obuf) - 2; m++)
-                obuf[m] = "01234567890\n"[rand() % 12];
+                obuf[m] = "0123456789\n"[rand() % 11];
             obuf[m++] = '\n';
             obuf[m]   = '\0';
 
@@ -105,7 +103,7 @@ int main(int argc, const char* argv[])
     }
 
     for (;;) {
-       if (CONN_Wait(conn, eIO_Read, timeout) != eIO_Success) {
+       if (CONN_Wait(conn, eIO_Read, net_info->timeout) != eIO_Success) {
             CONN_Close(conn);
             CORE_LOG(eLOG_Fatal, "Error waiting for reading");
         }
@@ -113,9 +111,10 @@ int main(int argc, const char* argv[])
         status = CONN_Read(conn, ibuf, sizeof(ibuf), &n, eIO_ReadPersist);
         if (n) {
             char* descr = CONN_Description(conn);
-            CORE_DATAF(ibuf, n, ("%lu bytes read from service (%s%s%s):",
-                                 (unsigned long) n, CONN_GetType(conn),
-                                 descr ? ", " : "", descr ? descr : ""));
+            CORE_DATAF_EX(eLOG_Note, ibuf, n,
+                          ("%lu bytes read from service (%s%s%s):",
+                           (unsigned long) n, CONN_GetType(conn),
+                           descr ? ", " : "", descr ? descr : ""));
             if (descr)
                 free(descr);
         }
@@ -126,6 +125,8 @@ int main(int argc, const char* argv[])
             break;
         }
     }
+
+    ConnNetInfo_Destroy(net_info);
     CONN_Close(conn);
 
 #if 0
@@ -153,121 +154,11 @@ int main(int argc, const char* argv[])
         CORE_LOG(eLOG_Fatal, "Error reading from service ID1");
     }
 
-    
-    ConnNetInfo_Destroy(net_info);
     CORE_LOGF(eLOG_Note, ("%d bytes read from service ID1", n));
     CONN_Close(conn);
 #endif
 
+    CORE_LOG(eLOG_Note, "TEST completed successfully");
     CORE_SetLOG(0);
     return 0/*okay*/;
 }
-
-
-/*
- * --------------------------------------------------------------------------
- * $Log: test_ncbi_service_connector.c,v $
- * Revision 6.34  2005/07/15 21:16:03  lavr
- * Fix int<->size_t mismatch bug causing 64-bit builds to check lamely
- *
- * Revision 6.33  2005/07/11 18:25:36  lavr
- * Spell ADDEND; better testing w/o ugly hardcoded pattern
- *
- * Revision 6.32  2005/05/02 16:12:43  lavr
- * Use global random seed
- *
- * Revision 6.31  2005/01/28 17:44:48  lavr
- * Fix:  forgotten to merge status variable relocation
- *
- * Revision 6.30  2005/01/28 17:41:22  lavr
- * Allow CONN_TIMEOUT from the environment to be used in Wait-on-Read
- *
- * Revision 6.29  2004/02/23 15:23:43  lavr
- * New (last) parameter "how" added in CONN_Write() API call
- *
- * Revision 6.28  2003/10/21 11:37:47  lavr
- * Set write timeout from the environment/registry instead of explicitly
- *
- * Revision 6.27  2003/07/24 16:38:59  lavr
- * Add conditional check for early connection drops (#if 0'd)
- *
- * Revision 6.26  2003/05/29 18:03:49  lavr
- * Extend read error message with a reason (if any)
- *
- * Revision 6.25  2003/05/14 03:58:43  lavr
- * Match changes in respective APIs of the tests
- *
- * Revision 6.24  2003/05/05 20:31:23  lavr
- * Add date/time stamp to each log message printed
- *
- * Revision 6.23  2003/04/04 21:01:06  lavr
- * Modify readout procedure
- *
- * Revision 6.22  2002/10/28 15:47:12  lavr
- * Use "ncbi_ansi_ext.h" privately and use strncpy0()
- *
- * Revision 6.21  2002/09/24 15:10:09  lavr
- * Fix test not to dereference NULL pointer resulting from failed connection
- *
- * Revision 6.20  2002/08/07 16:38:08  lavr
- * EIO_ReadMethod enums changed accordingly; log moved to end
- *
- * Revision 6.19  2002/03/22 19:47:41  lavr
- * Test_assert.h made last among the include files
- *
- * Revision 6.18  2002/03/21 22:02:16  lavr
- * Change default server from "ray" into "www.ncbi.nlm.nih.gov"
- *
- * Revision 6.17  2002/02/05 21:45:55  lavr
- * Included header files rearranged
- *
- * Revision 6.16  2002/01/16 21:23:15  vakatov
- * Utilize header "test_assert.h" to switch on ASSERTs in the Release mode too
- *
- * Revision 6.15  2001/09/24 20:36:22  lavr
- * Adjusted parameters in SERVICE_CreateConnectorEx()
- *
- * Revision 6.14  2001/06/11 22:17:28  lavr
- * Wait-for-reading timeout made finite
- *
- * Revision 6.13  2001/06/07 17:53:47  lavr
- * Persistent reading from test connection
- *
- * Revision 6.12  2001/06/01 16:19:10  lavr
- * Added (ifdef'ed out) an internal test connection to service ID1
- *
- * Revision 6.11  2001/05/11 16:05:41  lavr
- * Change log message corrected
- *
- * Revision 6.10  2001/05/11 15:38:01  lavr
- * Print connector type along with read data
- *
- * Revision 6.9  2001/04/24 21:42:43  lavr
- * Brushed code to use CORE_LOG facility only.
- *
- * Revision 6.8  2001/01/25 17:13:22  lavr
- * Added: close/free everything on program exit: useful to check memory leaks
- *
- * Revision 6.7  2001/01/23 23:22:34  lavr
- * debug_printout was given an enum value (instead of "boolean" 1)
- *
- * Revision 6.6  2001/01/09 15:35:20  lavr
- * Removed header <unistd.h>, unknown on WinNT
- *
- * Revision 6.5  2001/01/08 23:13:19  lavr
- * C/C++ -> C only
- *
- * Revision 6.4  2001/01/08 22:42:42  lavr
- * Further development of the test-suite
- *
- * Revision 6.3  2001/01/03 22:40:24  lavr
- * Minor adjustment
- *
- * Revision 6.2  2000/12/29 18:25:27  lavr
- * More tests added (still not yet complete).
- *
- * Revision 6.1  2000/10/20 17:31:07  lavr
- * Initial revision
- *
- * ==========================================================================
- */
